"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.NextAuthHandler = NextAuthHandler;

var _logger = _interopRequireWildcard(require("../utils/logger"));

var _detectHost = require("../utils/detect-host");

var routes = _interopRequireWildcard(require("./routes"));

var _pages = _interopRequireDefault(require("./pages"));

var _init = require("./init");

var _assert = require("./lib/assert");

var _cookie = require("./lib/cookie");

var _cookie2 = require("cookie");

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

async function getBody(req) {
  try {
    return await req.json();
  } catch (_unused) {}
}

async function toInternalRequest(req) {
  if (req instanceof Request) {
    var _req$headers$get, _url$searchParams$get, _headers$xForwarded;

    const url = new URL(req.url);
    const nextauth = url.pathname.split("/").slice(3);
    const headers = Object.fromEntries(req.headers.entries());
    const query = Object.fromEntries(url.searchParams.entries());
    query.nextauth = nextauth;
    return {
      action: nextauth[0],
      method: req.method,
      headers,
      body: await getBody(req),
      cookies: (0, _cookie2.parse)((_req$headers$get = req.headers.get("cookie")) !== null && _req$headers$get !== void 0 ? _req$headers$get : ""),
      providerId: nextauth[1],
      error: (_url$searchParams$get = url.searchParams.get("error")) !== null && _url$searchParams$get !== void 0 ? _url$searchParams$get : nextauth[1],
      host: (0, _detectHost.detectHost)((_headers$xForwarded = headers["x-forwarded-host"]) !== null && _headers$xForwarded !== void 0 ? _headers$xForwarded : headers.host),
      query
    };
  }

  return req;
}

async function NextAuthHandler(params) {
  var _req$body$callbackUrl, _req$body, _req$query2, _req$body2, _req$body3;

  const {
    options: userOptions,
    req: incomingRequest
  } = params;
  const req = await toInternalRequest(incomingRequest);
  (0, _logger.setLogger)(userOptions.logger, userOptions.debug);
  const assertionResult = (0, _assert.assertConfig)({
    options: userOptions,
    req
  });

  if (Array.isArray(assertionResult)) {
    assertionResult.forEach(_logger.default.warn);
  } else if (assertionResult instanceof Error) {
    var _req$query;

    const {
      pages,
      theme
    } = userOptions;

    _logger.default.error(assertionResult.code, assertionResult);

    const authOnErrorPage = (pages === null || pages === void 0 ? void 0 : pages.error) && req.action === "signin" && ((_req$query = req.query) === null || _req$query === void 0 ? void 0 : _req$query.callbackUrl.startsWith(pages.error));

    if (!(pages !== null && pages !== void 0 && pages.error) || authOnErrorPage) {
      if (authOnErrorPage) {
        _logger.default.error("AUTH_ON_ERROR_PAGE_ERROR", new Error(`The error page ${pages === null || pages === void 0 ? void 0 : pages.error} should not require authentication`));
      }

      const render = (0, _pages.default)({
        theme
      });
      return render.error({
        error: "configuration"
      });
    }

    return {
      redirect: `${pages.error}?error=Configuration`
    };
  }

  const {
    action,
    providerId,
    error,
    method = "GET"
  } = req;
  const {
    options,
    cookies
  } = await (0, _init.init)({
    userOptions,
    action,
    providerId,
    host: req.host,
    callbackUrl: (_req$body$callbackUrl = (_req$body = req.body) === null || _req$body === void 0 ? void 0 : _req$body.callbackUrl) !== null && _req$body$callbackUrl !== void 0 ? _req$body$callbackUrl : (_req$query2 = req.query) === null || _req$query2 === void 0 ? void 0 : _req$query2.callbackUrl,
    csrfToken: (_req$body2 = req.body) === null || _req$body2 === void 0 ? void 0 : _req$body2.csrfToken,
    cookies: req.cookies,
    isPost: method === "POST"
  });
  const sessionStore = new _cookie.SessionStore(options.cookies.sessionToken, req, options.logger);

  if (method === "GET") {
    const render = (0, _pages.default)({ ...options,
      query: req.query,
      cookies
    });
    const {
      pages
    } = options;

    switch (action) {
      case "providers":
        return await routes.providers(options.providers);

      case "session":
        {
          const session = await routes.session({
            options,
            sessionStore
          });
          if (session.cookies) cookies.push(...session.cookies);
          return { ...session,
            cookies
          };
        }

      case "csrf":
        return {
          headers: [{
            key: "Content-Type",
            value: "application/json"
          }],
          body: {
            csrfToken: options.csrfToken
          },
          cookies
        };

      case "signin":
        if (pages.signIn) {
          let signinUrl = `${pages.signIn}${pages.signIn.includes("?") ? "&" : "?"}callbackUrl=${encodeURIComponent(options.callbackUrl)}`;
          if (error) signinUrl = `${signinUrl}&error=${encodeURIComponent(error)}`;
          return {
            redirect: signinUrl,
            cookies
          };
        }

        return render.signin();

      case "signout":
        if (pages.signOut) return {
          redirect: pages.signOut,
          cookies
        };
        return render.signout();

      case "callback":
        if (options.provider) {
          const callback = await routes.callback({
            body: req.body,
            query: req.query,
            headers: req.headers,
            cookies: req.cookies,
            method,
            options,
            sessionStore
          });
          if (callback.cookies) cookies.push(...callback.cookies);
          return { ...callback,
            cookies
          };
        }

        break;

      case "verify-request":
        if (pages.verifyRequest) {
          return {
            redirect: pages.verifyRequest,
            cookies
          };
        }

        return render.verifyRequest();

      case "error":
        if (["Signin", "OAuthSignin", "OAuthCallback", "OAuthCreateAccount", "EmailCreateAccount", "Callback", "OAuthAccountNotLinked", "EmailSignin", "CredentialsSignin", "SessionRequired"].includes(error)) {
          return {
            redirect: `${options.url}/signin?error=${error}`,
            cookies
          };
        }

        if (pages.error) {
          return {
            redirect: `${pages.error}${pages.error.includes("?") ? "&" : "?"}error=${error}`,
            cookies
          };
        }

        return render.error({
          error: error
        });

      default:
    }
  } else if (method === "POST") {
    switch (action) {
      case "proxy":
        {
          const cookies = {};

          switch ((_req$body3 = req.body) === null || _req$body3 === void 0 ? void 0 : _req$body3.action) {
            case "signin":
              {
                var _csrfTokenRes$cookies, _csrfTokenRes$cookies2, _req$body4, _req$body5, _signInRes$cookies, _signInRes$cookies$fi, _signInRes$cookies2, _signInRes$cookies2$f;

                const csrfTokenRes = await NextAuthHandler({
                  req: {
                    host: req.host,
                    action: "csrf"
                  },
                  options: userOptions
                });
                const csrfToken = csrfTokenRes.body.csrfToken;
                const csrfTokenCookie = (_csrfTokenRes$cookies = csrfTokenRes.cookies) === null || _csrfTokenRes$cookies === void 0 ? void 0 : (_csrfTokenRes$cookies2 = _csrfTokenRes$cookies.find(c => c.name === options.cookies.csrfToken.name)) === null || _csrfTokenRes$cookies2 === void 0 ? void 0 : _csrfTokenRes$cookies2.value;

                if (csrfTokenCookie) {
                  cookies[options.cookies.csrfToken.name] = csrfTokenCookie;
                }

                const callbackUrl = (_req$body4 = req.body) === null || _req$body4 === void 0 ? void 0 : _req$body4.callbackUrl;
                cookies[options.cookies.callbackUrl.name] = callbackUrl;
                const signInRes = await NextAuthHandler({
                  req: {
                    host: req.host,
                    action: "signin",
                    method: "POST",
                    cookies,
                    body: {
                      csrfToken,
                      callbackUrl
                    },
                    providerId: (_req$body5 = req.body) === null || _req$body5 === void 0 ? void 0 : _req$body5.providerId
                  },
                  options: userOptions
                });
                const authorizationUrl = signInRes.redirect;
                const url = new URL(authorizationUrl);
                const params = new URLSearchParams(url.search);
                const stateEncrypted = (_signInRes$cookies = signInRes.cookies) === null || _signInRes$cookies === void 0 ? void 0 : (_signInRes$cookies$fi = _signInRes$cookies.find(c => c.name === options.cookies.state.name)) === null || _signInRes$cookies$fi === void 0 ? void 0 : _signInRes$cookies$fi.value;
                const state = params.get("state");
                const codeChallenge = params.get("code_challenge");
                const codeVerifier = (_signInRes$cookies2 = signInRes.cookies) === null || _signInRes$cookies2 === void 0 ? void 0 : (_signInRes$cookies2$f = _signInRes$cookies2.find(c => c.name === options.cookies.pkceCodeVerifier.name)) === null || _signInRes$cookies2$f === void 0 ? void 0 : _signInRes$cookies2$f.value;
                const provider = options.providers.find(({
                  id
                }) => {
                  var _req$body6;

                  return id === ((_req$body6 = req.body) === null || _req$body6 === void 0 ? void 0 : _req$body6.providerId);
                });
                let clientId;

                if ((provider === null || provider === void 0 ? void 0 : provider.type) === "oauth") {
                  clientId = provider.clientId;
                }

                return {
                  headers: [{
                    key: "Content-Type",
                    value: "application/json"
                  }],
                  body: {
                    state,
                    stateEncrypted,
                    codeVerifier,
                    codeChallenge,
                    clientId
                  }
                };
              }

            case "callback":
              {
                var _req$body7, _req$body8, _req$body9, _req$body10, _req$body11, _callbackRes$cookies, _callbackRes$cookies$;

                if ((_req$body7 = req.body) !== null && _req$body7 !== void 0 && _req$body7.codeVerifier) {
                  cookies[options.cookies.pkceCodeVerifier.name] = req.body.codeVerifier;
                }

                if ((_req$body8 = req.body) !== null && _req$body8 !== void 0 && _req$body8.stateEncrypted) {
                  cookies[options.cookies.state.name] = req.body.stateEncrypted;
                }

                const callbackRes = await NextAuthHandler({
                  req: {
                    host: req.host,
                    action: "callback",
                    cookies,
                    providerId: (_req$body9 = req.body) === null || _req$body9 === void 0 ? void 0 : _req$body9.providerId,
                    query: {
                      state: (_req$body10 = req.body) === null || _req$body10 === void 0 ? void 0 : _req$body10.state,
                      code: (_req$body11 = req.body) === null || _req$body11 === void 0 ? void 0 : _req$body11.code
                    }
                  },
                  options: userOptions
                });
                const location = callbackRes.redirect;
                const url = new URL(location);
                const params = new URLSearchParams(url.search);
                const error = params.get("error");

                if (error) {
                  return {
                    body: {
                      error
                    }
                  };
                }

                const sessionToken = (_callbackRes$cookies = callbackRes.cookies) === null || _callbackRes$cookies === void 0 ? void 0 : (_callbackRes$cookies$ = _callbackRes$cookies.find(c => c.name === options.cookies.sessionToken.name)) === null || _callbackRes$cookies$ === void 0 ? void 0 : _callbackRes$cookies$.value;
                return {
                  body: {
                    sessionToken
                  }
                };
              }

            case "signout":
              {
                var _req$body12, _csrfTokenRes$cookies3, _csrfTokenRes$cookies4;

                if ((_req$body12 = req.body) !== null && _req$body12 !== void 0 && _req$body12.sessionToken) {
                  cookies[options.cookies.sessionToken.name] = req.body.sessionToken;
                }

                const csrfTokenRes = await NextAuthHandler({
                  req: {
                    host: req.host,
                    action: "csrf"
                  },
                  options: userOptions
                });
                const csrfToken = csrfTokenRes.body.csrfToken;
                const csrfTokenCookie = (_csrfTokenRes$cookies3 = csrfTokenRes.cookies) === null || _csrfTokenRes$cookies3 === void 0 ? void 0 : (_csrfTokenRes$cookies4 = _csrfTokenRes$cookies3.find(c => c.name === options.cookies.csrfToken.name)) === null || _csrfTokenRes$cookies4 === void 0 ? void 0 : _csrfTokenRes$cookies4.value;

                if (csrfTokenCookie) {
                  cookies[options.cookies.csrfToken.name] = csrfTokenCookie;
                }

                await NextAuthHandler({
                  req: {
                    host: req.host,
                    action: "signout",
                    method: "POST",
                    cookies,
                    body: {
                      csrfToken
                    }
                  },
                  options: userOptions
                });
                return {
                  body: {
                    signedOut: true
                  }
                };
              }

            case "session":
              {
                var _req$body13;

                if ((_req$body13 = req.body) !== null && _req$body13 !== void 0 && _req$body13.sessionToken) {
                  cookies[options.cookies.sessionToken.name] = req.body.sessionToken;
                }

                return await NextAuthHandler({
                  req: {
                    host: req.host,
                    cookies,
                    action: "session"
                  },
                  options: userOptions
                });
              }

            case "providers":
              {
                return await NextAuthHandler({
                  req: {
                    action: "providers"
                  },
                  options: userOptions
                });
              }
          }

          return {
            status: 400
          };
        }

      case "signin":
        if (options.csrfTokenVerified && options.provider) {
          const signin = await routes.signin({
            query: req.query,
            body: req.body,
            options
          });
          if (signin.cookies) cookies.push(...signin.cookies);
          return { ...signin,
            cookies
          };
        }

        return {
          redirect: `${options.url}/signin?csrf=true`,
          cookies
        };

      case "signout":
        if (options.csrfTokenVerified) {
          const signout = await routes.signout({
            options,
            sessionStore
          });
          if (signout.cookies) cookies.push(...signout.cookies);
          return { ...signout,
            cookies
          };
        }

        return {
          redirect: `${options.url}/signout?csrf=true`,
          cookies
        };

      case "callback":
        if (options.provider) {
          if (options.provider.type === "credentials" && !options.csrfTokenVerified) {
            return {
              redirect: `${options.url}/signin?csrf=true`,
              cookies
            };
          }

          const callback = await routes.callback({
            body: req.body,
            query: req.query,
            headers: req.headers,
            cookies: req.cookies,
            method,
            options,
            sessionStore
          });
          if (callback.cookies) cookies.push(...callback.cookies);
          return { ...callback,
            cookies
          };
        }

        break;

      case "_log":
        if (userOptions.logger) {
          try {
            var _req$body14;

            const {
              code,
              level,
              ...metadata
            } = (_req$body14 = req.body) !== null && _req$body14 !== void 0 ? _req$body14 : {};

            _logger.default[level](code, metadata);
          } catch (error) {
            _logger.default.error("LOGGER_ERROR", error);
          }
        }

        return {};

      default:
    }
  }

  return {
    status: 400,
    body: `Error: This action with HTTP ${method} is not supported by NextAuth.js`
  };
}