"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.SessionProvider = SessionProvider;
exports.apiBaseUrl = apiBaseUrl;
exports.fetchData = fetchData;
exports.getProviders = getProviders;
exports.getSession = getSession;
exports.getSignInInfo = getSignInInfo;
exports.signIn = signIn;
exports.signOut = signOut;
exports.useSession = useSession;

var React = _interopRequireWildcard(require("react"));

var _utils = require("../client/_utils");

var _logger2 = _interopRequireWildcard(require("../utils/logger"));

var _parseUrl = _interopRequireDefault(require("../utils/parse-url"));

var SecureStore = _interopRequireWildcard(require("expo-secure-store"));

var _reactNative = require("react-native");

require("react-native-url-polyfill/auto");

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

const storageKeys = {
  sessionToken: "next-auth.sessionToken"
};

function apiBaseUrl(__NEXTAUTH) {
  return `${__NEXTAUTH.baseUrlServer}${__NEXTAUTH.basePathServer}`;
}

async function fetchData(path, __NEXTAUTH, logger, params) {
  const url = `${apiBaseUrl(__NEXTAUTH)}/proxy`;
  const sessionToken = await SecureStore.getItemAsync(storageKeys.sessionToken);

  try {
    const options = {
      method: "POST",
      headers: {
        "Content-Type": "application/json"
      },
      body: JSON.stringify({
        action: path,
        sessionToken,
        ...params
      })
    };
    const res = await fetch(url, options);
    const data = await res.json();
    if (!res.ok) throw data;
    return Object.keys(data).length > 0 ? data : null;
  } catch (error) {
    logger.error("CLIENT_FETCH_ERROR", {
      error: error,
      url
    });
    return null;
  }
}

async function getSignInInfo({
  provider,
  proxyRedirectUri
}) {
  return await fetchData("signin", __NEXTAUTH, logger, {
    providerId: provider,
    callbackUrl: proxyRedirectUri
  });
}

const nextAuthUrl = "https://staging.learncontigo.com/api/auth/mobile";
const __NEXTAUTH = {
  baseUrl: (0, _parseUrl.default)(nextAuthUrl).origin,
  basePath: (0, _parseUrl.default)(nextAuthUrl).path,
  baseUrlServer: (0, _parseUrl.default)(nextAuthUrl).origin,
  basePathServer: (0, _parseUrl.default)(nextAuthUrl).path,
  _lastSync: 0,
  _session: undefined,
  _getSession: () => {}
};
const logger = (0, _logger2.proxyLogger)(_logger2.default, __NEXTAUTH.basePath);
const SessionContext = React.createContext(undefined);

function useSession(options) {
  const value = React.useContext(SessionContext);

  if (!value && process.env.NODE_ENV !== "production") {
    throw new Error("[next-auth]: `useSession` must be wrapped in a <SessionProvider />");
  }

  const {
    onUnauthenticated
  } = options !== null && options !== void 0 ? options : {};
  const unauthenticated = value.status === "unauthenticated";
  React.useEffect(() => {
    if (unauthenticated) {
      if (onUnauthenticated) onUnauthenticated();
    }
  }, [unauthenticated, onUnauthenticated]);
  return value;
}

async function getSession() {
  const session = await fetchData("session", __NEXTAUTH, logger);
  return session;
}

async function getProviders() {
  return await fetchData("providers", __NEXTAUTH, logger);
}

async function signIn(initiateExpoAuthFlow, options) {
  const signinResult = await initiateExpoAuthFlow();

  if (signinResult === null) {
    return;
  }

  const {
    result,
    state,
    codeVerifier,
    stateEncrypted,
    provider
  } = signinResult;
  if (result.type !== "success") return;
  const data = await fetchData("callback", __NEXTAUTH, logger, {
    providerId: provider,
    code: result.params.code,
    state,
    stateEncrypted,
    codeVerifier
  });

  if (!data) {
    _reactNative.Alert.alert("Error", "Callback error.");

    return;
  }

  const {
    error,
    sessionToken
  } = data;

  if (!!error || !sessionToken) {
    switch (error) {
      case "OAuthAccountNotLinked":
        {
          _reactNative.Alert.alert("Error", "The email associated with the account you just logged in is being used by another account. Please log into that account and then link to other providers afterwards.");

          break;
        }
    }

    return;
  }

  if (sessionToken) {
    await SecureStore.setItemAsync(storageKeys.sessionToken, sessionToken);
    await __NEXTAUTH._getSession({
      event: "storage"
    });
  }
}

async function signOut() {
  const data = await fetchData("signout", __NEXTAUTH, logger);

  if (data) {
    await SecureStore.deleteItemAsync(storageKeys.sessionToken);
    await __NEXTAUTH._getSession({
      event: "storage"
    });
  }
}

function SessionProvider(props) {
  const {
    children,
    basePath,
    refetchInterval
  } = props;
  if (basePath) __NEXTAUTH.basePath = basePath;
  const hasInitialSession = props.session !== undefined;
  __NEXTAUTH._lastSync = hasInitialSession ? (0, _utils.now)() : 0;
  const [session, setSession] = React.useState(() => {
    if (hasInitialSession) __NEXTAUTH._session = props.session;
    return props.session;
  });
  const [loading, setLoading] = React.useState(!hasInitialSession);
  React.useEffect(() => {
    __NEXTAUTH._getSession = async ({
      event
    } = {}) => {
      try {
        const storageEvent = event === "storage";

        if (storageEvent || __NEXTAUTH._session === undefined) {
          __NEXTAUTH._lastSync = (0, _utils.now)();
          __NEXTAUTH._session = await getSession();
          setSession(__NEXTAUTH._session);
          return;
        }

        if (!event || __NEXTAUTH._session === null || (0, _utils.now)() < __NEXTAUTH._lastSync) {
          return;
        }

        __NEXTAUTH._lastSync = (0, _utils.now)();
        __NEXTAUTH._session = await getSession();
        setSession(__NEXTAUTH._session);
      } catch (error) {
        logger.error("CLIENT_SESSION_ERROR", error);
      } finally {
        setLoading(false);
      }
    };

    __NEXTAUTH._getSession();

    return () => {
      __NEXTAUTH._lastSync = 0;
      __NEXTAUTH._session = undefined;

      __NEXTAUTH._getSession = () => {};
    };
  }, []);
  React.useEffect(() => {
    if (refetchInterval) {
      const refetchIntervalTimer = setInterval(() => {
        if (__NEXTAUTH._session) {
          __NEXTAUTH._getSession({
            event: "poll"
          });
        }
      }, refetchInterval * 1000);
      return () => clearInterval(refetchIntervalTimer);
    }
  }, [refetchInterval]);
  const value = React.useMemo(() => ({
    data: session,
    status: loading ? "loading" : session ? "authenticated" : "unauthenticated"
  }), [session, loading]);
  return <SessionContext.Provider value={value}>{children}</SessionContext.Provider>;
}